#pragma once

#include <utils.h>
#include "WindowMapping.h"
#include "LayerVariable.h"
#include "Serializable.h"

#include "Transform.h"

#define GRID_SIZE 1.0f

#define MAX_VARIABLES 10

class BaseRenderable: public Serializable
{
public:
	BaseRenderable(int TypeID);
	~BaseRenderable(void);

	void Translate(float delta_x, float delta_y);
	void Scale(float x, float y);

	void SaveToXml(TiXmlElement *currentElement);
	void LoadFromXml(TiXmlElement *currentElement);

	virtual void ChangeParameter(int parameterIndex, float newParameter);
	virtual void IncrementParameter(int parameterIndex, float deltaParameter);

	void InitializeBase();
	void AssignLayerToVariable(int variableIndex, Renderable2d* subLayer);
	int m_variableCount;
	LayerVariable m_layerVariables[MAX_VARIABLES];
	
	virtual void Render() = 0;
	virtual void Update(bool force) = 0;
	int id;
	int m_typeID;

	BaseRenderable *ParentLayer;

protected:
	virtual void SetVariables() = 0;
	virtual void Initialize() = 0;

	Transform m_transform;

	//WindowMapping m_windowMapping;
	LayerManager* m_layerManager;

	//physical bitmap
	GLubyte* m_bitmapBits;

	//GL texture!
	GLuint m_Texture;

private:
	static int lastID;
};