#include "Renderable3d.h"

GLubyte SomeColors[5][3] = {
	{0x88, 0, 0},
	{0, 0x88, 0},
	{0, 0, 0x88},
	{0x44, 0x44, 0},
	{0x44, 0, 0x44}
};

Renderable3d::Renderable3d(int typeID) : BaseRenderable(typeID)
{
	m_WindowSizeX = m_WindowSizeY = 5;
	world = World::theWorld;
	m_entityCollection.Clear();
	for(int i = 0; i < TEXTURE_COUNT; i++)
	{
		//generate textures
		glGenTextures(TEXTURE_COUNT, m_entityTextures);
	}

	SetSpaceWindow();
}

void Renderable3d::Render()
{
	//ONLY draw, ffs.
    glClearColor (0.5,0.5,0.6,1.0);
    glClear (GL_COLOR_BUFFER_BIT);

	m_entityCollection.Render(m_entityTextures);
}

void Renderable3d::Update(bool force)
{
	bool changedWindow = SetSpaceWindow();
	if(!changedWindow && !force) return;

	//set texture
	for(int i = 0; i < TEXTURE_COUNT; i++)
	{
		glBindTexture(GL_TEXTURE_2D, m_entityTextures[i]);
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	// Linear Min Filter
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// Linear Mag Filter

		glTexImage2D(GL_TEXTURE_2D, 0, 3, 1, 1, 0, GL_RGB, GL_UNSIGNED_BYTE, SomeColors[i]);

	}

	//building 3d entity collection
	BuildCollection();

}

bool Renderable3d::SetSpaceWindow()
{
	//find size of space window
	float x_width = DivFloat(m_WindowSizeX, GRID_SIZE);
	float y_width = DivFloat(m_WindowSizeY, GRID_SIZE);

	//keep divisible by 2
	if(ModFloat(x_width, 2.0) != 0)
	{
		x_width += GRID_SIZE;
	}

	if(ModFloat(y_width, 2.0) != 0)
	{
		y_width += GRID_SIZE;
	}

	//find center of screen
	float center_x = DivFloat(world->camera.Lookat.x, (x_width/8) * GRID_SIZE);
	float center_y = DivFloat(world->camera.Lookat.y, (y_width/8) * GRID_SIZE);

	WindowMapping tempWindow;
	tempWindow.SetPhysicalWindow(m_WindowSizeX, m_WindowSizeY);
	tempWindow.SetWorldWindow(-y_width/2 + center_y, x_width/2 + center_x, -x_width/2 + center_x, y_width/2 + center_y);

	//no changes - nothing should happen. Window is snapped to grid points.
	if(tempWindow == m_spaceWindow)
		return false;

	//copy constructor
	m_spaceWindow = tempWindow;
	return true;
}

bool Renderable3d::SetWindowSize(int newWindowSize)
{
	if(newWindowSize >=1 && m_WindowSizeX != newWindowSize)
	{
		m_WindowSizeX = m_WindowSizeY = newWindowSize;
		return true;
	}

	return false;
}